/*********************************************************************************
 * Copyright (c) 2006-2008 Forschungszentrum Juelich GmbH 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Forschungszentrum Juelich GmbH nor the names of its 
 * contributors may be used to endorse or promote products derived from this 
 * software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 ********************************************************************************/

package pl.edu.icm.unicore.uvos.server;

import java.security.cert.CertificateException;
import java.security.cert.CertificateExpiredException;
import java.security.cert.X509Certificate;

import javax.net.ssl.X509TrustManager;

import org.apache.log4j.Logger;
import org.mortbay.log.Log;

import eu.unicore.security.CertificateUtils;

/**
 * <p>
 * AuthSSLX509TrustManager is decorator of other X509TrustManager.
 * Performs CRL checks and some additional logging.
 * </p>
 * TODO This class is more or less copied from gateway. UNICORE should use a common implementation.
 */
public class AuthSSLX509TrustManager implements X509TrustManager
{
	private static final Logger LOG = Logger.getLogger(AuthSSLX509TrustManager.class);
	private X509TrustManager defaultTrustManager = null;

	public AuthSSLX509TrustManager(final X509TrustManager defaultTrustManager)
	{
		if (defaultTrustManager == null)
			throw new IllegalArgumentException(
					"Trust manager may not be null");
		this.defaultTrustManager = defaultTrustManager;
	}

	public void checkClientTrusted(X509Certificate[] certificates, String s)
			throws CertificateException
	{
		if (LOG.isDebugEnabled() && certificates != null)
		{
			for (int c = 0; c < certificates.length; c++)
			{
				X509Certificate cert = certificates[c];
				LOG.debug(" Client certificate:");
				LOG.debug("  Subject DN: "
						+ cert.getSubjectDN());
				LOG.debug("  Issuer: " + cert.getIssuerDN());
			}
		}

		try
		{
			defaultTrustManager.checkClientTrusted(certificates, s);
			CertificateUtils.verifyCertificate(certificates, true, false);
		} catch (CertificateExpiredException iae)
		{
			if (LOG.isDebugEnabled())
			{
				String msg = "Certificate " + CertificateUtils.safePrintSubject(certificates)
					+ " is no longer valid. ";

				Throwable cause = iae.getCause();
				if (cause != null)
				{
					msg += cause.getMessage() == null ? cause.toString() : cause.getMessage();
				} else
					msg += iae.getMessage() == null ? "" : iae.getMessage();
				//log it here as jetty doesn't log it really nicely.
				LOG.debug("Connection refused: " + msg);
			}
			throw new CertificateException(iae);
		} catch (CertificateException ce)
		{
			String msg = "The supplied certificate " + 
				CertificateUtils.safePrintSubject(certificates)	+ " is invalid: " + 
				ce.getMessage() == null ? "" : ce.getMessage();
			Log.debug("Connection refused: " + msg);
			throw new CertificateException(msg);
		}
	}

	/**
	 * @see X509TrustManager#isServerTrusted(X509Certificate[])
	 */
	public void checkServerTrusted(X509Certificate[] certificates, String s)
			throws CertificateException
	{
		defaultTrustManager.checkServerTrusted(certificates, s);
		CertificateUtils.verifyCertificate(certificates, true, false);
	}

	/**
	 * @see X509TrustManager#getAcceptedIssuers()
	 */
	public X509Certificate[] getAcceptedIssuers()
	{
		return this.defaultTrustManager.getAcceptedIssuers();
	}
}
