/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Nov 30, 2009
 * Author: T. Rekawek <newton@mat.uni.torun.pl>
 */

package pl.edu.icm.unicore.uvos.server;

import java.io.FileInputStream;
import java.net.InetAddress;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.UnknownHostException;
import java.security.KeyStore;
import java.security.PrivateKey;
import java.security.cert.Certificate;
import java.security.cert.X509Certificate;

import org.apache.log4j.Logger;

import pl.edu.icm.unicore.uvos.util.ConfDefaults;
import pl.edu.icm.unicore.uvos.util.Configuration;

/**
 * Implementation of CryptoDataProvider
 * @author K. Benedyczak
 */
public class CryptoDataProviderImpl implements CryptoDataProvider
{
	private static final Logger log = Logger.getLogger(CryptoDataProviderImpl.class);
	private X509Certificate[] serverCertificate;
	private transient PrivateKey privKey;
	private String issuerURI;
	private Configuration config;

	public CryptoDataProviderImpl(Configuration config) throws Exception
	{
		this.config = config;
		readServerIdentity();
	}
	
	private void readServerIdentity() throws Exception
	{
		String ks = config.getAsString(ConfDefaults.KEYSTORE, 
				ConfDefaults.keystore, log);
		String ksPasswd = config.getAsString(ConfDefaults.KEYSTORE_PASSWD, 
				ConfDefaults.keystorePasswd, log);
		String ksType = config.getAsString(ConfDefaults.KEYSTORE_TYPE,
				ConfDefaults.keystoreType, log);
		String ksAlias = config.getAsString(ConfDefaults.KEYSTORE_ALIAS,
				ConfDefaults.keystoreAlias, log);
		String ksKeyPasswd = config.getAsString(ConfDefaults.KEYSTORE_KEY_PASSWD,
				ksPasswd, log, false);
		KeyStore keystore = KeyStore.getInstance(ksType);
		FileInputStream fis = new FileInputStream(ks);
		try{
			keystore.load(fis, ksPasswd.toCharArray());
		}
		finally{
			fis.close();
		}
		if (!keystore.containsAlias(ksAlias))
			throw new Exception("Keystore " + ks + " doesn't contain alias " + ksAlias);
		if (!keystore.isKeyEntry(ksAlias))
			throw new Exception(ksAlias + " is trusted certificate entry, while should be" +
					" a private key and cerificate entry.");

		Certificate[] raw = keystore.getCertificateChain(ksAlias);

		serverCertificate = new X509Certificate[raw.length];
		for (int i=0; i<raw.length; i++)
			serverCertificate[i] = (X509Certificate)raw[i];
		privKey = (PrivateKey)keystore.getKey(ksAlias, ksKeyPasswd.toCharArray());
		issuerURI = establishServerURI();
	}
	
	public String getIssuerURI()
	{
		return issuerURI;
	}

	public PrivateKey getPrivateKey()
	{
		return privKey;
	}

	public X509Certificate getServerCertificate()
	{
		return serverCertificate[0];
	}

	public X509Certificate[] getServerCertificateC()
	{
		return serverCertificate;
	}
	
	/**
	 * Tries to establish an URL which will identify this server. It will be used
	 * (if not overriden by config option) in Issuer element of SAML Respones.
	 * <p>
	 * Warning: it is not guaranteed to succeed - it is much safer to set the 
	 * URI by hand in config file.
	 */
	private String establishServerURI()
	{
		String uri = config.getAsString(ConfDefaults.ISSUER_URI, 
				ConfDefaults.defIssuerURI, null, false);
		if (uri.equals(ConfDefaults.defIssuerURI))
		{
			log.info("No " + ConfDefaults.ISSUER_URI + " is set. " +
					"Will try to guess it." );
		} else
		{
			try
			{
				new URI(uri);
				return uri;
			} catch (URISyntaxException e)
			{
				log.info(ConfDefaults.ISSUER_URI + 
					" is set but is invalid (isn't a URI): " +
					e.getMessage() + ". Will try to guess it." );
			}
		}
		
		String fqdn;
		boolean ssl = config.getAsBool(ConfDefaults.HTTPS_ENABLE, 
				ConfDefaults.httpsEnable, null);
		try
		{
			InetAddress addr = InetAddress.getLocalHost();
			fqdn = addr.getCanonicalHostName();
		} catch (UnknownHostException e)
		{
			log.warn("Can't determine host domain name " +
				"and the host URI isn't specified. " +
				"Will do the best to produce the the URL but" +
				"it may be incorrect.");
			if (ssl)
				fqdn = config.getAsString(ConfDefaults.HTTPS_HOST, 
						ConfDefaults.httpsHost, null);
			else
				fqdn = config.getAsString(ConfDefaults.HTTP_HOST, 
						ConfDefaults.httpHost, null);
		}
		
		int port = -1;
		if (ssl)
			port = config.getAsInt(ConfDefaults.HTTPS_PORT, 
					ConfDefaults.httpsPort, null);
		else
			port =	config.getAsInt(ConfDefaults.HTTP_PORT, 
					ConfDefaults.httpPort, null);
		uri = "http://" + fqdn + ":" + port; 
		log.info("Will use autogenerated server's URI: " + uri);
		return uri;  
	}
}
