/*********************************************************************************
 * Copyright (c) 2008 Forschungszentrum Juelich GmbH 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Forschungszentrum Juelich GmbH nor the names of its 
 * contributors may be used to endorse or promote products derived from this 
 * software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 ********************************************************************************/
package pl.edu.icm.unicore.uvos.server;

import java.io.IOException;
import java.io.InputStream;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.security.KeyStore;
import java.security.SecureRandom;

import javax.net.ssl.HandshakeCompletedEvent;
import javax.net.ssl.HandshakeCompletedListener;
import javax.net.ssl.KeyManager;
import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLPeerUnverifiedException;
import javax.net.ssl.SSLSocket;
import javax.net.ssl.TrustManager;
import javax.net.ssl.TrustManagerFactory;
import javax.net.ssl.X509TrustManager;

import org.apache.log4j.Logger;
import org.mortbay.jetty.security.SslSelectChannelConnector;
import org.mortbay.resource.Resource;


/**
 * Slight extension to the Jetty SslSocketConnector, allowing to customise trust
 * management. Will also log the address of the remote host trying to 
 * establish a connection.
 */
public class CustomSslSocketConnector extends SslSelectChannelConnector
{
	private final static Logger log = Logger.getLogger(CustomSslSocketConnector.class);

	private transient char[]truststorePasswd;
	private transient char[]keystorePasswd;
	private transient char[]keyPasswd;
	
	public CustomSslSocketConnector(char[] truststorePasswd,
			char[] keystorePasswd, char[] keyPasswd)
	{
		super();
		this.truststorePasswd = truststorePasswd;
		this.keystorePasswd = keystorePasswd;
		this.keyPasswd = keyPasswd;
	}

	@Override
	protected void configure(Socket socket) throws IOException
	{
		InetSocketAddress peer = (InetSocketAddress) socket.getRemoteSocketAddress();
		if (log.isDebugEnabled() && peer != null
				&& peer.getAddress() != null)
		{
			log.debug("Connection attempt from "
					+ peer.getAddress().getHostAddress());
			if (socket instanceof SSLSocket)
			{
				SSLSocket ssl = (SSLSocket) socket;
				ssl.addHandshakeCompletedListener(new HandshakeCompletedListener()
				{
					public void handshakeCompleted(HandshakeCompletedEvent hce)
					{
						String msg;
						try
						{
							msg = String.valueOf(hce.getPeerPrincipal());
						} catch (SSLPeerUnverifiedException spe)
						{
							msg = "Problem: Unverified peer!";
						}
						log.debug("SSL connection to <" + msg + "> established.");
					}
				});
			}
		}
		super.configure(socket);
	}
	
	protected SSLContext createSSLContext() throws Exception
	{
	        SecureRandom secureRandom= getSecureRandomAlgorithm() == null ? 
	        		null : SecureRandom.getInstance(getSecureRandomAlgorithm());
	        SSLContext context = getProvider() == null ? 
	        		SSLContext.getInstance(getProtocol()) :
	        		SSLContext.getInstance(getProtocol(), getProvider());
	        context.init(getKeyManagers(), getTrustManagers(), secureRandom);
	        return context;

	}
	
	protected KeyManager[] getKeyManagers() throws Exception
	{
		InputStream keystoreInputStream = null;
		KeyManager[] keyManagers=null;
		KeyStore keyStore = null;
		try
		{
			if (getKeystore() != null)
			{
				keystoreInputStream = Resource.newResource(getKeystore()).getInputStream();
				keyStore = KeyStore.getInstance(getKeystoreType());
				keyStore.load(keystoreInputStream, keystorePasswd);
			}
		}
		finally
		{
			if (keystoreInputStream != null)
				keystoreInputStream.close();
		}

		KeyManagerFactory keyManagerFactory = KeyManagerFactory.getInstance(getSslKeyManagerFactoryAlgorithm());
		keyManagerFactory.init(keyStore, keyPasswd==null ? keystorePasswd : keyPasswd);
		keyManagers=keyManagerFactory.getKeyManagers();
		return keyManagers;
	}

	protected TrustManager[] getTrustManagers() throws Exception
	{
		TrustManager[] trustManagers = null;
		InputStream truststoreInputStream = null;
		KeyStore trustStore = null;
	        try
	        {
	            if (getTruststore() != null)
	            {
	                truststoreInputStream = Resource.newResource(getTruststore()).getInputStream();
	                trustStore = KeyStore.getInstance(getTruststoreType());
	                trustStore.load(truststoreInputStream, truststorePasswd);
	            }
	        }
	        finally
	        {
	            if (truststoreInputStream != null)
	                truststoreInputStream.close();
	        }

	        TrustManagerFactory trustManagerFactory = TrustManagerFactory.getInstance(getSslTrustManagerFactoryAlgorithm());
		trustManagerFactory.init(trustStore);
		trustManagers = trustManagerFactory.getTrustManagers();
		TrustManager[] res=new X509TrustManager[trustManagers.length];
		for (int i=0;i<res.length;i++)
		{
			res[i] = trustManagers[i] instanceof X509TrustManager ? 
					new AuthSSLX509TrustManager((X509TrustManager)trustManagers[i]) 
					: trustManagers[i];
		}
		return res;
	}
}