/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Aug 8, 2007
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.server;

import org.apache.log4j.Logger;
import org.codehaus.xfire.transport.http.XFireServlet;
import org.mortbay.component.LifeCycle;
import org.mortbay.component.LifeCycle.Listener;
import org.mortbay.jetty.Connector;
import org.mortbay.jetty.Handler;
import org.mortbay.jetty.Server;
import org.mortbay.jetty.SessionIdManager;
import org.mortbay.jetty.deployer.WebAppDeployer;
import org.mortbay.jetty.handler.ContextHandlerCollection;
import org.mortbay.jetty.handler.DefaultHandler;
import org.mortbay.jetty.handler.HandlerCollection;
import org.mortbay.jetty.handler.RequestLogHandler;
import org.mortbay.jetty.nio.SelectChannelConnector;
import org.mortbay.jetty.security.SslSelectChannelConnector;
import org.mortbay.jetty.servlet.Context;
import org.mortbay.jetty.servlet.HashSessionIdManager;
import org.picocontainer.PicoContainer;

import eu.unicore.security.CertificateUtils;

import pl.edu.icm.unicore.uvos.util.ConfDefaults;
import pl.edu.icm.unicore.uvos.util.Configuration;

/**
 * Jetty server implementation. Here appropriate connectors are configured.
 * 
 * This module is not managed by Pico container as it depends on it. The dependency is required
 * to setup web applications (which may not use Pico at all, and e.g. Spring container).
 * Also we don't wan't to put PicoContiner in it as it is a bad practice.
 * @author K. Benedyczak
 */
public class JettyServer
{
	private static final Logger log = Logger.getLogger(JettyServer.class);
	public static final String UVOS_SERVER_CONTAINER_REFERENCE = "uvos.container.ref";
	private Server jettyServer;
	private Configuration conf;
	
	public JettyServer(Configuration conf, final PicoContainer pico) throws Exception
	{
		this.conf = conf;
		Connector[] conns = getConnectors();
		//System.setProperty("DEBUG", "true");
		jettyServer = new Server();
		jettyServer.setConnectors(conns);
		
		final ContextHandlerCollection contexts = new ContextHandlerCollection();
		RequestLogHandler requestLogHandler = new RequestLogHandler();

		Context root=new Context(jettyServer, "/", Context.SESSIONS);
		root.addServlet(XFireServlet.class, "/*");

		HandlerCollection handlers = new HandlerCollection();
		handlers.setHandlers(new Handler[]{
				contexts,
				root,
				new DefaultHandler(),
				requestLogHandler});
		jettyServer.setHandler(handlers);

		String crlProps = System.getProperty("crlmanager.properties.file");
		if(crlProps!=null)
		{
			log.info("Initialising CRL checker using configuration " + crlProps);
			try
			{
				Class.forName(CertificateUtils.class.getName());
			}catch(Exception ex){
				log.error("Error setting up CRL checker.", ex);
			}
		}
		
		contexts.addLifeCycleListener(new Listener()
		{
			public void lifeCycleStarting(LifeCycle arg0)
			{
				log.debug("Initializing UVOS web applications contexts with Pico reference");
				Handler[] handlers = contexts.getHandlers();
				if (handlers == null)
					return;
				for (Handler h: handlers)
				{
					log.debug("Web app: " + h);
					Context c = (Context)h;
					c.setAttribute(UVOS_SERVER_CONTAINER_REFERENCE, 
						pico);
				}
			}
			
			public void lifeCycleStarted(LifeCycle arg0) {}
			public void lifeCycleFailure(LifeCycle arg0, Throwable arg1) {}
			public void lifeCycleStopping(LifeCycle arg0) {}			
			public void lifeCycleStopped(LifeCycle arg0) {}			
		});
		WebAppDeployer deployer = new WebAppDeployer();
		deployer.setContexts(contexts);
		deployer.setWebAppDir(conf.getAsString(ConfDefaults.WEBAPPS_DIR, 
				ConfDefaults.webappsDir, log));
		deployer.setParentLoaderPriority(false);
		deployer.setExtract(true);
		deployer.setAllowDuplicates(false);
		jettyServer.addLifeCycle(deployer);
		
		if (conf.getAsBool(ConfDefaults.HTTPS_FAST_RANDOM, ConfDefaults.httpsFastRandom, log))
		{
			SessionIdManager sm = new HashSessionIdManager(new java.util.Random());
			jettyServer.setSessionIdManager(sm);
		}
	}
	
	public String getConnectorsDesc()
	{
		StringBuilder sb = new StringBuilder();
		for (Connector c: jettyServer.getConnectors())
		{
			sb.append(c.getHost());
			sb.append(":");
			sb.append(c.getLocalPort());
			sb.append(" ");
		}
		return sb.toString();
	}
	
	public String[] getURLs()
	{
		String[] ret = new String[jettyServer.getConnectors().length];
		for (int i=0; i<ret.length; i++)
		{
			StringBuilder sb = new StringBuilder();
			Connector c = jettyServer.getConnectors()[i];
			if (c instanceof SslSelectChannelConnector)
				sb.append("https://");
			else
				sb.append("http://");
			sb.append(c.getHost());
			sb.append(":");
			sb.append(c.getLocalPort());
			ret[i] = sb.toString();
		}
		return ret;
	}
	
	private Connector[] getConnectors() throws Exception
	{
		boolean plain = conf.getAsBool(ConfDefaults.HTTP_ENABLE, 
				ConfDefaults.httpEnable, log);
		boolean ssl = conf.getAsBool(ConfDefaults.HTTPS_ENABLE, 
				ConfDefaults.httpsEnable, log);
		if (!(plain || ssl))
			throw new Exception("No one connector is enabled");				
		Connector []ret;
		if (plain && ssl)
			ret = new Connector[2];
		else
			ret = new Connector[1];
		int i=0;
		
		if (plain)
			ret[i++] = getPlainConnector();
		if (ssl)
			ret[i++] = getSSLConnector();
		return ret;
	}

	private Connector getPlainConnector()
	{
		Connector connector = new SelectChannelConnector();
		connector.setPort(conf.getAsInt(ConfDefaults.HTTP_PORT, 
				ConfDefaults.httpPort, log));
		connector.setHost(conf.getAsString(ConfDefaults.HTTP_HOST, 
				ConfDefaults.httpHost, log));
		return connector;
	}

	private Connector getSSLConnector()
	{
		String tsPasswd = conf.getAsString(ConfDefaults.TRUSTSTORE_PASSWD, 
				ConfDefaults.truststorePasswd, log);
		String ksPasswd = conf.getAsString(ConfDefaults.KEYSTORE_PASSWD, 
				ConfDefaults.keystorePasswd, log);
		String keyPasswd = conf.getAsString(ConfDefaults.KEYSTORE_KEY_PASSWD, 
				ksPasswd, log, false);
		
		CustomSslSocketConnector connector = new CustomSslSocketConnector(
				tsPasswd.toCharArray(), ksPasswd.toCharArray(), keyPasswd.toCharArray());
		
		connector.setPort(conf.getAsInt(ConfDefaults.HTTPS_PORT, 
				ConfDefaults.httpsPort, log));
		connector.setHost(conf.getAsString(ConfDefaults.HTTPS_HOST, 
				ConfDefaults.httpsHost, log));
		
		connector.setWantClientAuth(true);
		connector.setNeedClientAuth(!conf.getAsBool(ConfDefaults.HTTPS_ANON, 
				ConfDefaults.httpsAnon, log));
		
		connector.setTrustPassword(tsPasswd);
		connector.setTruststore(conf.getAsString(ConfDefaults.TRUSTSTORE,
				ConfDefaults.truststore, log));
		connector.setTruststoreType(conf.getAsString(ConfDefaults.TRUSTSTORE_TYPE,
				ConfDefaults.truststoreType, log));
		
		connector.setPassword(ksPasswd);
		connector.setKeyPassword(keyPasswd);
		connector.setKeystore(conf.getAsString(ConfDefaults.KEYSTORE,
				ConfDefaults.keystore, log));
		connector.setKeystoreType(conf.getAsString(ConfDefaults.KEYSTORE_TYPE,
				ConfDefaults.keystoreType, log));
		
		return connector;
	}

	
	public void start() throws Exception 
	{
		jettyServer.start();
	}
	
	public void stop() throws Exception 
	{
		jettyServer.stop();
	}
}
